/*
  OLED.cpp - Library for using the NewHaven OLED Character Display:NHD-0220DZW-AG5 & NHD-0420DZW-AG5
  Created by Scott Swaaley, November 25, 2013
  Released into the public domain.
*/

#include "Arduino.h"
#include "OLED.h"

byte SCL_PIN; // serial clock
byte SDOUT_PIN; // output to OLED (SDIN on OLED Datasheet)
byte CS_PIN; // chip select (active low)
byte commandDelay = 0;
byte scrollLineAddress;

const byte COMMAND = 0;
const byte DATA = 1;

const byte L1C1 = B10000000; // command byte to set ddram address to Line 1, Character 1
const byte L2C1 = B11000000; // command byte to set ddram address to Line 2, Character 1
const byte L3C1 = B10010100; // command byte to set ddram address to Line 3, Character 1
const byte L4C1 = B11010100; // command byte to set ddram address to Line 4, Character 1


OLED::OLED(byte _SCL_PIN, byte _SDOUT_PIN, byte _CS_PIN) {
	setPins(_SCL_PIN, _SDOUT_PIN, _CS_PIN);
}

void OLED::setPins(byte _SCL_PIN, byte _SDOUT_PIN, byte _CS_PIN) {
	
	SCL_PIN = _SCL_PIN;
	SDOUT_PIN = _SDOUT_PIN;
	CS_PIN = _CS_PIN;
	
	pinMode(SCL_PIN,OUTPUT);
	pinMode(SDOUT_PIN,OUTPUT);
	pinMode(CS_PIN,OUTPUT);

}

void OLED::begin() {
	init();

}

void OLED::init() {
	writeCommand(B00111000); //function set: 8-bit, English/Japanese Font Table
	delay(commandDelay);
	writeCommand(B00000000);//display off
	delay(commandDelay);
	writeCommand(B00000001);//clear display
	delay(commandDelay);
	writeCommand(B0000110);//entry mode set
	delay(commandDelay);
	writeCommand(B00000010);//return home
	delay(commandDelay);
	writeCommand(B00001100);//display on, cursor off, cursor not blinking
	delay(commandDelay);
	writeCommand(L1C1);// set ddram address
	delay(commandDelay);
}

void OLED::writeCommand(unsigned char ins) {
  unsigned char i;
  preWrite(COMMAND);
  for(i=0x80;i;i>>=1) {
    if((ins&0x80)==0x80) digitalWrite(SDOUT_PIN,HIGH);
    else digitalWrite(SDOUT_PIN,LOW);
    ins=(ins<<1);
    toggleClock();
  }
  postWrite();	 
}

void OLED::print(byte lineNumber, String contentString) {
  if(lineNumber > 0) writeCommand(getAddress(lineNumber,1));
  preWrite(DATA);

  char contentChars[contentString.length()+1];
  contentString.toCharArray(contentChars, contentString.length()+1);

  for(int charIndex = 0; charIndex<20;charIndex++) {
    unsigned char dat;
    if(charIndex < contentString.length()) dat = contentChars[charIndex];
    else dat = B00100000;    
    for (byte i=0x80;i;i>>=1) {
      if((dat&0x80)==0x80) digitalWrite(SDOUT_PIN,HIGH);
      else digitalWrite(SDOUT_PIN,LOW);
      dat=(dat<<1);
      toggleClock();
    }
  }
  postWrite();
}

void OLED::print(byte lineNumber,char contentChar) {
  if(lineNumber > 0) writeCommand(getAddress(lineNumber,1));
  preWrite(DATA);
  for (byte i=0x80;i;i>>=1) {
    if((contentChar&0x80)==0x80) digitalWrite(SDOUT_PIN,HIGH);
    else digitalWrite(SDOUT_PIN,LOW);
    contentChar=(contentChar<<1);
    toggleClock();
  }
  postWrite();
}

void OLED::toggleClock() {
  digitalWrite(SCL_PIN,LOW);
  digitalWrite(SCL_PIN,HIGH);
  digitalWrite(SCL_PIN,LOW);
}

//  sent before data or command, tells the OLED if data or commands are coming.
void OLED::preWrite(byte writeType) {
  digitalWrite(CS_PIN,LOW);
  digitalWrite(SCL_PIN,LOW);
  digitalWrite(SDOUT_PIN,writeType);
  toggleClock();
  digitalWrite(SDOUT_PIN,LOW);
  toggleClock();
}

void OLED::postWrite() {
  digitalWrite(CS_PIN,HIGH);
  digitalWrite(SCL_PIN,HIGH);
}


int charIndex = 0;
//int charAddress;// = B10010011;
float lastScroll = 0;
int scrollDelay;
char contentChars[250];
int contentStringLength = 0;

void OLED::setScroll(byte lineNumber, int _scrollDelay, String contentString) {
  scrollLineAddress = getAddress(lineNumber,20);
  contentStringLength = contentString.length();
  contentString.toCharArray(contentChars, contentStringLength + 1);
  scrollDelay = _scrollDelay;
}

void OLED::scrollLine() {
  writeCommand(B0000110); // sets entry mode
  if(millis() - lastScroll > scrollDelay) {
    byte charAddress = scrollLineAddress;
    int _charIndex = charIndex;
    for(int i = 0; i < 20 ; i++) {
      
      writeCommand(charAddress--);
      
      if(_charIndex < 0 ) print(0,' ');
      else if(_charIndex > contentStringLength - 1)  print(0,' ');
      else print(0,contentChars[_charIndex]);
      _charIndex--;
    }
    charIndex++;
    
    if(charAddress < 0x80) charAddress = scrollLineAddress;
    if(charIndex > contentStringLength + 19) charIndex = 0;
    lastScroll = millis();
    
  }
}

byte addressLookup[4][20] = //line number, character number
	{{B10000000,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,B10010011},
	 {B11000000,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,B11010011},
	 {B10010100,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,B10100111},
	 {B11010100,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,B11100111}};

byte OLED::getAddress(byte lineNumber, byte charNumber) {
	return addressLookup[lineNumber-1][charNumber-1];
}


